@@ -191,7 +191,7 @@ class READ_INFO: public Load_data_param
     For example, suppose we have an ujis file with bytes 0x8FA10A, where:
     - 0x8FA1 is an incomplete prefix of a 3-byte character
       (it should be [8F][A1-FE][A1-FE] to make a full 3-byte character)
-    - 0x0A is a line demiliter
+    - 0x0A is a line delimiter
     This file has some broken data, the trailing [A1-FE] is missing.
 
     In this example it works as follows:
@@ -419,9 +419,9 @@ int mysql_load(THD *thd, const sql_exchange *ex, TABLE_LIST *table_list,
                                     thd->lex->first_select_lex()->leaf_tables,
                                     FALSE,
                                     INSERT_ACL | UPDATE_ACL,
-                                    INSERT_ACL | UPDATE_ACL, FALSE))
+                                    INSERT_ACL | UPDATE_ACL, false))
      DBUG_RETURN(-1);
-  if (!table_list->table ||               // do not suport join view
+  if (!table_list->table ||               // do not support join view
       !table_list->single_table_updatable() || // and derived tables
       check_key_in_view(thd, table_list))
   {
@@ -684,9 +684,14 @@ int mysql_load(THD *thd, const sql_exchange *ex, TABLE_LIST *table_list,
     thd->abort_on_warning= !ignore && thd->is_strict_mode();
     thd->get_stmt_da()->reset_current_row_for_warning(1);
 
-    if (prepare_for_replace(table, info.handle_duplicates, info.ignore))
-      DBUG_RETURN(1);
-
+    bool create_lookup_handler= handle_duplicates != DUP_ERROR;
+    if ((table_list->table->file->ha_table_flags() & HA_DUPLICATE_POS))
+    {
+      create_lookup_handler= true;
+      if ((error= table_list->table->file->ha_rnd_init_with_error(0)))
+        goto err;
+    }
+    table->file->prepare_for_modify(true, create_lookup_handler);
     thd_progress_init(thd, 2);
     fix_rownum_pointers(thd, thd->lex->current_select, &info.copied);
     if (table_list->table->validate_default_values_of_unset_fields(thd))
@@ -707,8 +712,8 @@ int mysql_load(THD *thd, const sql_exchange *ex, TABLE_LIST *table_list,
                             set_fields, set_values, read_info,
                             *ex->enclosed, skip_lines, ignore);
 
-    if (unlikely(finalize_replace(table, handle_duplicates, ignore)))
-      DBUG_RETURN(1);
+    if (table_list->table->file->ha_table_flags() & HA_DUPLICATE_POS)
+      table_list->table->file->ha_rnd_end();
 
     thd_proc_info(thd, "End bulk insert");
     if (likely(!error))
@@ -856,6 +861,8 @@ err:
               thd->transaction->stmt.modified_non_trans_table);
   table->file->ha_release_auto_increment();
   table->auto_increment_field_not_null= FALSE;
+  if (thd->tmp_table_binlog_handled)
+    table->mark_as_not_binlogged(); 		// tmp table changes are not in binlog
   thd->abort_on_warning= 0;
   DBUG_RETURN(error);
 }
@@ -991,6 +998,7 @@ static bool write_execute_load_query_log_event(THD *thd, const sql_exchange* ex,
   if (!(load_data_query= (char *)thd->strmake(query_str.ptr(), query_str.length())))
     return TRUE;
 
+  thd->tmp_table_binlog_handled= 1;
   Execute_load_query_log_event
     e(thd, load_data_query, query_str.length(),
       (uint) (fname_start - 1), (uint) fname_end,
@@ -1003,7 +1011,7 @@ static bool write_execute_load_query_log_event(THD *thd, const sql_exchange* ex,
 #endif
 
 /****************************************************************************
-** Read of rows of fixed size + optional garage + optonal newline
+** Read of rows of fixed size + optional garage + optional newline
 ****************************************************************************/
 
 static int
@@ -1015,7 +1023,7 @@ read_fixed_length(THD *thd, COPY_INFO &info, TABLE_LIST *table_list,
   List_iterator_fast<Item> it(fields_vars);
   Item *item;
   TABLE *table= table_list->table;
-  bool err, progress_reports;
+  bool err= false, progress_reports;
   ulonglong counter, time_to_report_progress;
   DBUG_ENTER("read_fixed_length");
 
@@ -1025,8 +1033,6 @@ read_fixed_length(THD *thd, COPY_INFO &info, TABLE_LIST *table_list,
   if ((thd->progress.max_counter= read_info.file_length()) == ~(my_off_t) 0)
     progress_reports= 0;
 
-  Write_record write(thd, table, &info, NULL);
-
   while (!read_info.read_fixed_length())
   {
     if (thd->killed)
@@ -1077,7 +1083,7 @@ read_fixed_length(THD *thd, COPY_INFO &info, TABLE_LIST *table_list,
         uchar save_chr;
         if ((length=(uint) (read_info.row_end - pos)) > fixed_length)
           length= fixed_length;
-        save_chr= pos[length]; pos[length]= '\0'; // Safeguard aganst malloc
+        save_chr= pos[length]; pos[length]= '\0'; // Safeguard against malloc
         dst->load_data_set_value(thd, (const char *) pos, length, &read_info);
         pos[length]= save_chr;
         if ((pos+= length) > read_info.row_end)
@@ -1093,10 +1099,11 @@ read_fixed_length(THD *thd, COPY_INFO &info, TABLE_LIST *table_list,
                           thd->get_stmt_da()->current_row_for_warning());
     }
 
+    bool trg_skip_row= false;
     if (thd->killed ||
         fill_record_n_invoke_before_triggers(thd, table, set_fields, set_values,
                                              ignore_check_option_errors,
-                                             TRG_EVENT_INSERT))
+                                             TRG_EVENT_INSERT, &trg_skip_row))
       DBUG_RETURN(1);
 
     switch (table_list->view_check_option(thd, ignore_check_option_errors)) {
@@ -1107,7 +1114,8 @@ read_fixed_length(THD *thd, COPY_INFO &info, TABLE_LIST *table_list,
       DBUG_RETURN(-1);
     }
 
-    err= write.write_record();
+    if (!trg_skip_row)
+      err= write_record(thd, table, &info);
     table->auto_increment_field_not_null= FALSE;
     if (err)
       DBUG_RETURN(1);
@@ -1156,8 +1164,6 @@ read_sep_field(THD *thd, COPY_INFO &info, TABLE_LIST *table_list,
   if ((thd->progress.max_counter= read_info.file_length()) == ~(my_off_t) 0)
     progress_reports= 0;
 
-  Write_record write(thd, table, &info, NULL);
-
   for (;;it.rewind())
   {
     if (thd->killed)
@@ -1243,13 +1249,21 @@ read_sep_field(THD *thd, COPY_INFO &info, TABLE_LIST *table_list,
       }
     }
 
+    bool trg_skip_row= false;
     if (unlikely(thd->killed) ||
         unlikely(fill_record_n_invoke_before_triggers(thd, table, set_fields,
                                                       set_values,
                                                       ignore_check_option_errors,
-                                                      TRG_EVENT_INSERT)))
+                                                      TRG_EVENT_INSERT,
+                                                      &trg_skip_row)))
       DBUG_RETURN(1);
 
+    if (trg_skip_row)
+    {
+      read_info.next_line();
+      continue;
+    }
+
     switch (table_list->view_check_option(thd,
                                           ignore_check_option_errors)) {
     case VIEW_CHECK_SKIP:
@@ -1259,7 +1273,7 @@ read_sep_field(THD *thd, COPY_INFO &info, TABLE_LIST *table_list,
       DBUG_RETURN(-1);
     }
 
-    err= write.write_record();
+    err= write_record(thd, table, &info);
     table->auto_increment_field_not_null= FALSE;
     if (err)
       DBUG_RETURN(1);
@@ -1303,9 +1317,7 @@ read_xml_field(THD *thd, COPY_INFO &info, TABLE_LIST *table_list,
   DBUG_ENTER("read_xml_field");
   
   no_trans_update_stmt= !table->file->has_transactions_and_rollback();
-
-  Write_record write(thd, table, &info, NULL);
-
+  
   for ( ; ; it.rewind())
   {
     bool err;
@@ -1367,12 +1379,19 @@ read_xml_field(THD *thd, COPY_INFO &info, TABLE_LIST *table_list,
 
     DBUG_ASSERT(!item);
 
+    bool trg_skip_row= false;
     if (thd->killed ||
         fill_record_n_invoke_before_triggers(thd, table, set_fields, set_values,
                                              ignore_check_option_errors,
-                                             TRG_EVENT_INSERT))
+                                             TRG_EVENT_INSERT, &trg_skip_row))
       DBUG_RETURN(1);
 
+    if (trg_skip_row)
+    {
+      read_info.next_line();
+      continue;
+    }
+
     switch (table_list->view_check_option(thd,
                                           ignore_check_option_errors)) {
     case VIEW_CHECK_SKIP:
@@ -1382,7 +1401,7 @@ read_xml_field(THD *thd, COPY_INFO &info, TABLE_LIST *table_list,
       DBUG_RETURN(-1);
     }
     
-    err= write.write_record();
+    err= write_record(thd, table, &info);
     table->auto_increment_field_not_null= false;
     if (err)
       DBUG_RETURN(1);
@@ -1404,7 +1423,7 @@ read_xml_field(THD *thd, COPY_INFO &info, TABLE_LIST *table_list,
 char
 READ_INFO::unescape(char chr)
 {
-  /* keep this switch synchornous with the ESCAPE_CHARS macro */
+  /* keep this switch synchronous with the ESCAPE_CHARS macro */
   switch(chr) {
   case 'n': return '\n';
   case 't': return '\t';
@@ -1455,7 +1474,7 @@ READ_INFO::READ_INFO(THD *thd, File file_par,
   uint length= MY_MAX(charset()->mbmaxlen, MY_MAX(m_field_term.length(),
                                                   m_line_term.length())) + 1;
   set_if_bigger(length,line_start.length());
-  stack= stack_pos= (int*) thd->alloc(sizeof(int) * length);
+  stack= stack_pos= thd->alloc<int>(length);
 
   DBUG_ASSERT(m_fixed_length < UINT_MAX32);
   if (data.reserve((size_t) m_fixed_length))
